package deriveum.flows;

import co.paralleluniverse.fibers.Suspendable;
import com.google.common.collect.ImmutableList;
import deriveum.contracts.CDSContract;
import deriveum.states.CDSState;
import deriveum.states.TokenState;
import net.corda.core.contracts.Command;
import net.corda.core.contracts.ContractState;
import net.corda.core.contracts.StateAndRef;
import net.corda.core.contracts.UniqueIdentifier;
import net.corda.core.crypto.SecureHash;
import net.corda.core.flows.*;
import net.corda.core.identity.Party;
import net.corda.core.node.services.Vault;
import net.corda.core.node.services.vault.QueryCriteria;
import net.corda.core.transactions.SignedTransaction;
import net.corda.core.transactions.TransactionBuilder;
import net.corda.core.utilities.ProgressTracker;

import java.security.PublicKey;
import java.util.Date;
import java.util.List;

import static net.corda.core.contracts.ContractsDSL.requireThat;

// -> Insurer
public class TerminateDueToExpiration {
    @InitiatingFlow
    @StartableByRPC
    public static class Initiator extends FlowLogic<SignedTransaction> {
        private Party issuer;
        private UniqueIdentifier contractId;

        public Initiator(Party issuer, UniqueIdentifier contractId) {
            this.contractId = contractId;
            this.issuer = issuer;
        }

        @Suspendable
        @Override
        public SignedTransaction call() throws FlowException {
            QueryCriteria.LinearStateQueryCriteria inputCriteria = new QueryCriteria.LinearStateQueryCriteria(null, ImmutableList.of(contractId), Vault.StateStatus.UNCONSUMED, null);
            StateAndRef inputStateAndRef = getServiceHub().getVaultService().queryBy(CDSState.class, inputCriteria).getStates().get(0);
            final Party notary = getServiceHub().getNetworkMapCache().getNotaryIdentities().get(0);

            CDSState cdsState = (CDSState) inputStateAndRef.getState().getData();


            TokenState tokenState = new TokenState(issuer, cdsState.getInsurer(), cdsState.getAmount());

            List<PublicKey> requiredSigners = ImmutableList.of(issuer.getOwningKey(), cdsState.getInsurer().getOwningKey());
            Command command = new Command(new CDSContract.Commands.TerminateDueToExpiration(), requiredSigners);

            TransactionBuilder txBuilder = new TransactionBuilder(notary)
                    .addInputState(inputStateAndRef)
                    .addOutputState(tokenState, CDSContract.ID)
                    .addCommand(command);

            SignedTransaction partStx = getServiceHub().signInitialTransaction(txBuilder);
            FlowSession flowSession = initiateFlow(cdsState.getInsurer());
            FlowSession flowSession2 = initiateFlow(issuer);

            SignedTransaction fullyStx = subFlow(new CollectSignaturesFlow(partStx, ImmutableList.of(flowSession, flowSession2)));
            return subFlow(new FinalityFlow(fullyStx, ImmutableList.of(flowSession, flowSession2)));
        }

        @InitiatedBy(Initiator.class)
        public static class Responder extends FlowLogic<SignedTransaction> {
            private FlowSession counterpartySession;
            private SecureHash txWeJustSigned;

            public Responder(FlowSession counterpartySession) {
                this.counterpartySession = counterpartySession;
            }

            @Suspendable
            @Override
            public SignedTransaction call() throws FlowException {
                class SignTxFlow extends SignTransactionFlow {

                    private SignTxFlow(FlowSession flowSession, ProgressTracker progressTracker) {
                        super(flowSession, progressTracker);
                    }

                    @Override
                    protected void checkTransaction(SignedTransaction stx) {
                        txWeJustSigned = stx.getId();
                    }
                }
                counterpartySession.getCounterpartyFlowInfo().getFlowVersion();
                SignTxFlow signTxFlow = new SignTxFlow(counterpartySession, SignTransactionFlow.Companion.tracker());
                subFlow(signTxFlow);
                return subFlow(new ReceiveFinalityFlow(counterpartySession, txWeJustSigned));
            }
        }
    }
}
