package deriveum.states;

import com.google.common.collect.ImmutableList;
import deriveum.contracts.CDSContract;
import net.corda.core.contracts.BelongsToContract;
import net.corda.core.contracts.ContractState;
import net.corda.core.contracts.LinearState;
import net.corda.core.contracts.UniqueIdentifier;
import net.corda.core.identity.AbstractParty;
import net.corda.core.identity.Party;
import net.corda.core.serialization.ConstructorForDeserialization;
import org.jetbrains.annotations.NotNull;

import java.util.List;

@BelongsToContract(CDSContract.class)
public class CDSState implements ContractState, LinearState {
    private int amount;
    private int period;
    private int state;
    private long createdAtTimestamp;
    private Party insurer;
    private Party insured;
    private Party resolver;
    private Party verifier;
    private final UniqueIdentifier linearId;

    @ConstructorForDeserialization
    public CDSState(int amount, int period, int state, long createdAtTimestamp, Party insurer, Party insured,
                    Party resolver, Party verifier, UniqueIdentifier linearId) {
        this.amount = amount;
        this.period = period;
        this.state = state;
        this.createdAtTimestamp = createdAtTimestamp;
        this.insurer = insurer;
        this.insured = insured;
        this.resolver = resolver;
        this.verifier = verifier;
        this.linearId = linearId;
    }

    public long getCreatedAtTimestamp() {
        return createdAtTimestamp;
    }

    public int getAmount() {
        return amount;
    }

    public int getPeriod() {
        return period;
    }

    public Party getInsurer() {
        return insurer;
    }

    public Party getInsured() {
        return insured;
    }

    public Party getResolver() {
        return resolver;
    }

    public Party getVerifier() {
        return verifier;
    }

    public int getState() {
        return state;
    }

    @NotNull
    @Override
    public UniqueIdentifier getLinearId() {
        return this.linearId;
    }

    @NotNull
    @Override
    public List<AbstractParty> getParticipants() {
        return ImmutableList.of(insurer, insured, verifier, resolver);
    }

    public CDSState withInsuredAgreeOnTermination() {
        return new CDSState(amount, period, 2, createdAtTimestamp, insurer, insured, resolver, verifier, new UniqueIdentifier());
    }

    public CDSState withInsurerAgreeOnTermination() {
        return new CDSState(amount, period, 3, createdAtTimestamp, insurer, insured, resolver, verifier, new UniqueIdentifier());
    }

    public CDSState withBothAgreeOnTermination() {
        return new CDSState(amount, period, 4, createdAtTimestamp, insurer, insured, resolver, verifier, new UniqueIdentifier());
    }


}
